# We might be surprised to find 'data' in a .py file, but it's small, well-curated using source control,
# and according to Turing and von Neumann, what we call "code" is just "the data that configures the machine".
#
# Each language configured here just has a set of words that should always be written as the top / most frequent
# words in that language, even if they're not very prevalent in (say) Wikipedia.

"""These words should go at the top of the ranking table for their language."""
TOP_DATA_OVERRIDES = {
    "en": ["bye", "hello", "hey", "hi", "I", "me", "no", "ok", "okay",
           "please", "pls", "thank", "thanks", "tks", "ty", "yes", "you"],
    "es": ["hola", "gracias", "no"],
    "it": ["no"],
    "ja": ["本"],
    "pt": ["obrigada", "obrigado"],
    "tl": ["po"],
    "zh": ["你好", "谢谢"]
}

"""These words should go in the stated position in their ranking tables - before the top data overrides are included."""
RANKED_DATA_OVERRIDES = {
    'de': {'bitte': 5, 'wir': 20, 'uns': 21, 'habe': 25, 'anliegen': 28, 'mir': 29, 'ihnen': 30, 'dir': 36, 'gerne': 38,
           'vielen': 41, 'mitarbeiter': 42, 'zufrieden': 52, 'mein': 63, 'bin': 65, 'guten': 67, 'antworten': 68,
           'weiter': 71, 'meine': 72, 'vertrag': 73, 'länger': 74, 'daher': 76, 'verbinden': 77, 'fragen': 79,
           'hast': 80, 'leider': 81, 'geht': 83, 'unseren': 84, 'melden': 88, 'nun': 92, 'unsere': 96, 'leite': 97},
    'en': {'chat': 61, 'payment': 76, 'customer': 77, 'conversation': 89, 'email': 95, 'wait': 100},
    'es': {'servicio': 19, 'consulta': 30, 'cuenta': 33, 'nuestro': 37, 'cliente': 39, 'información': 40, 'línea': 41,
           'banca': 44, 'estamos': 48, 'puedo': 49, 'móvil': 50, 'bac': 51, 'atención': 54, 'opción': 55, 'menú': 59,
           'momento': 62, 'horario': 63, 'puedes': 65, 'bienvenido': 66, 'solicitud': 69, 'buenas': 70, 'encuesta': 72,
           'nos': 77, 'conversación': 79, 'consultas': 80, 'escribe': 83, 'requerida': 85, 'servicios': 87,
           'nuestros': 88, 'minutos': 89, 'puntos': 90, 'ayudarte': 91, 'hacer': 92, 'envío': 93, 'responde': 94,
           'quiero': 95, 'días': 96, 'horarios': 98, 'saldo': 99, 'posible': 100},
    'fr': {'vous': 1, 'votre': 5, 'nous': 17, 'demande': 22, 'merci': 23, 'suis': 25, 'numéro': 34, 'bienvenue': 39,
           'conseiller': 53, 'nos': 55, 'notre': 57, 'lundi': 58, 'bonne': 60, 'oui': 63, 'conseillers': 65,
           'besoin': 68, 'journée': 70, 'terminer': 71, 'conversation': 72, 'répondent': 74, 'fermer': 76,
           'intéresse': 77, 'réponse': 78, 'satisfait': 79, 'transmise': 80, 'téléphone': 82, 'dossier': 85,
           'communiquer': 89, 'comment': 91, 'cela': 98},
    'it': {'visualizzabile': 9, 'grafico': 10, 'elemento': 11, 'ti': 16, 'utente': 17, 'tua': 21, 'numero': 25,
           'grazie': 30, 'mi': 31, 'ciao': 32, 'consulente': 33, 'tuo': 34, 'vuoi': 35, 'puoi': 38, 'richiesta': 40,
           'avere': 41, 'informazioni': 44, 'bisogno': 47, 'vodafone': 48, 'posso': 51, 'ci': 55, 'vorrei': 56,
           'servizio': 57, 'assistenza': 59, 'linea': 60, 'risponderà': 63, 'operatore': 65, 'contratto': 66,
           'supporto': 69, 'ricevere': 70, 'possibile': 71, 'esperto': 73, 'aiutarti': 74, 'gestire': 78, 'digita': 79,
           'nostro': 80, 'direttamente': 81, 'clienti': 83, 'passarti': 84, 'domanda': 87, 'notifica': 88,
           'risposta': 90, 'conto': 91, 'scrivi': 92, 'ringrazio': 95, 'contatto': 98, 'telefono': 99, 'attivo': 100},
    'nl': {'onze': 17, 'ons': 18, 'snel': 29, 'heb': 30, 'doen': 34, 'bedankt': 38, 'bericht': 44, 'huisnummer': 48,
           'postcode': 49, 'wij': 50, 'vragen': 54, 'langer': 59, 'storingen': 60, 'wachttijd': 61, 'ervaren': 63,
           'beantwoorden': 66, 'klantenservice': 67, 'verbind': 74, 'onderhoud': 76, 'sneller': 80, 'hebt': 82,
           'gegevens': 89, 'verwacht': 93, 'dankjewel': 94, 'hiervoor': 95, 'helpen': 97, 'collega': 98, 'nodig': 99},
    'pt': {'cartão': 15, 'atendimento': 19, 'digite': 22, 'fatura': 25, 'meu': 30, 'sim': 31, 'momento': 33,
           'seguro': 36, 'aqui': 37, 'opção': 39, 'voltar': 41, 'oi': 46, 'aguarde': 49, 'informações': 55,
           'informe': 57, 'falar': 58, 'sou': 59, 'posso': 62, 'assistente': 67, 'bom': 68, 'estou': 69, 'opções': 70,
           'desejada': 72, 'serviço': 75, 'quero': 79, 'limite': 80, 'preciso': 81, 'porto': 82, 'pedido': 89,
           'caso': 91, 'contato': 92, 'cadastro': 94, 'boa': 96, 'conta': 97, 'certo': 98, 'precisa': 99},
    'ru': {'вам': 4, 'помочь': 8, 'пожалуйста': 9, 'мы': 10, 'вы': 14, 'заказ': 16, 'меня': 17, 'здравствуйте': 18,
           'могу': 19, 'вас': 22, 'вопрос': 23, 'выберите': 24, 'вопросы': 26, 'мне': 28, 'чат': 29, 'зовут': 30,
           'привет': 32, 'спасибо': 33, 'списка': 38, 'день': 40, 'добрый': 41, 'поддержки': 43, 'звонке': 44,
           'водитель': 45, 'будет': 46, 'нет': 47, 'человека': 48, 'решить': 51, 'еще': 52, 'выбрать': 54, 'ниже': 56,
           'попробуйте': 57, 'связаться': 59, 'ваш': 60, 'помощь': 61, 'заказа': 62, 'готовы': 64, 'ваше': 65,
           'сейчас': 66, 'временно': 70, 'бот': 74, 'возможность': 75, 'нам': 78, 'сегодня': 80, 'служба': 81,
           'минут': 83, 'поездке': 84, 'обращение': 85, 'бота': 86, 'наши': 87, 'свои': 88, 'номер': 89, 'оставили': 90,
           'специалисты': 91, 'вами': 93, 'крутые': 94, 'приостановили': 95, 'специалистами': 96, 'ответами': 97,
           'задаваемые': 98, 'нажмите': 99, 'водителя': 100}}



from lplangid import language_classifier as lc, count_utils as cu

# This is the very simplest way to share data structures between tests.
# I dare say it can become unsafe if you do bad things. So please don't.
# Or move to some kind of fixture structure. One that works with PyCharm and isn't too fiddly.
ALL_TERM_RANKS, ALL_CHAR_WEIGHTS = lc.prepare_scoring_tables()

TEST_TEXTS = [("This is English", "en"),
              ("Esto es español", "es"),
              ("Can you please thank you", "en"),
              ("Hi", "en"),
              ("Hello", "en"),
              ("No , I'm presently a Vodafone customer", "en"),
              ("Gracias", "es"),
              ("24 pls", "en"),
              ("Obrigada, bom dia!", "pt"),
              ("I agree", "en"),
              ("吸尘器", "zh"),
              ("掃除機が壊れた", "ja"),
              ("书", "zh"),
              ("本", "ja"),
              ("転送が完了するまでしばらくお待ちください", "ja"),
              #  ("Me interesa", "es"),  # This would be nice to catch and correct!
              ("Please", "en"),
              ("안녕하세요? 안녕하세요?", "ko"),
              ("123", None),
              ("ᵔ", None),
              ("ᵔ is a hat", "en"),
              ("呗", "zh"),
              ("アンケートを終了いたします。お問い合わせありがとうございました。", "ja"),
              ("wuglmkqufd mgackxzrfk ydcxscavcb kvbtwygmew cicdrlnrck pqrawbsrsc jjmwwxpwem exouqfmkuj wuugakutqw "
               "nyfjakojtm vwnhvanazz fogkjhovxw grdhnmejzk hgquytxtzk xscfjlmozv pbnxfqmshr afzglkacyb lsfwcbrqcf",
               None),
              ("<!EncryptedText ,,, even if followed", None),
              ("displayText more text", None),
              ("kindly update me once done", "en"),
              ("http://go.microsoft.com/fwlink/?LinkID=267510&clcid=0x409", None),
              ("国内感染確認１万1543人 (クルーズ船除く) 新型コロナ", "ja"),
              ("क्या हाल है", "hi"),
              ("saya bisa bicara bahasa", "id"),
              ('Metadata\n-----------\n\n{"splus":{"os":"android"}}\n\n	', None),
              ('https://downloads.anywhere.com is enough to block classification', None),
              ('I have questions about all..', 'en'),
              ('Tôi đã đăng ký được 25 ngày. Sao đến giờ vẫn chưa có phản hồi', 'vi'),
              ('por favor desactiva mi tarjeta', 'es'),
              ("Let's go!", 'en')
              ]


def test_language_classifier_cases():
    for text in TEST_TEXTS:
        winner = lc.get_winner(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, text[0])
        assert winner == text[1], f"Expected {text[1]} instead of {winner} for \"{text[0]}\""


def test_language_classifier_cases2():
    classifier = lc.RRCLanguageClassifier.default_instance()
    for text in TEST_TEXTS:
        winner = classifier.get_winner(text[0])
        assert winner == text[1], f"Expected {text[1]} instead of {winner} for \"{text[0]}\""


def test_classifier_instance():
    classifier = lc.RRCLanguageClassifier.default_instance()
    assert classifier.get_winner("This is English") == "en"
    assert classifier.get_winner_score("This is English")[0] == "en"
    assert classifier.get_language_scores("This is English")[0][0] == "en"


def test_get_winner_score_for_digit():
    ws = lc.get_winner_score(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, '1')
    assert ws == (None, 0.0)


def test_get_winner_margin():
    winner1, margin = lc.get_winner_margin(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, 'no')
    winner2, score = lc.get_winner_score(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, 'no')
    assert winner1 == winner2 == 'it'
    assert margin < score
    winner2, margin2 = lc.get_winner_margin(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, '안녕하세요')
    assert margin2 > margin


def test_invert_char_tables():
    lang_to_char_weights = {"en": {"e": 0.4, "a": 0.3, "t": 0.3}, "id": {"a": 0.4, "e": 0.3, "n": 0.3}}
    char_to_lang_weights = lc.invert_char_tables(lang_to_char_weights)
    assert char_to_lang_weights["a"][0][0] == "id"
    assert char_to_lang_weights["e"][0][0] == "en"
    assert len(char_to_lang_weights["t"]) == 1


def _test_large_precision():
    """Remove _ from the beginning of this and ask Dominic for test data if you want to try this for testing changes.

    Used initially to compare softmax with L1-norm for character score distribution.

    With L1:      Attempted 99670, correct with 95210. (P: 95.55, R: 95.21, F: 95.37)
    With SoftMax: Attempted 99665, correct with 95169. (P: 95.45, R: 95.17, F: 95.33)
    So they're almost identical, L1 slightly better on this dataset.

    After adding LivePerson terms into term ranks: Attempted 96397, correct with 94164. (P: 97.69, R: 0.9416, F: 95.89)
    """
    test_data = open("/Users/dwiddows/Data/wikipedia_sample.tsv")
    import csv
    reader = csv.reader(test_data, delimiter="\t")
    attempted = 0
    correct = 0
    total = 0
    for lang, text in reader:
        total += 1
        winner = lc.get_winner(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, text)
        if winner:
            attempted += 1
        if attempted and winner == lang:
            correct += 1
    print(f"Attempted: {attempted}, Correct: {correct}, Out of total: {total}")
    precision, recall, f_measure = cu.precision_recall_f(total, attempted, correct)
    print(f"Precision: {precision}, Recall: {recall}, F-measure: {f_measure}")


def test_sr_hr():
    """Shows that text from a Serbian radio station gets classified as Croatian,
    because of the vernacular latin orthography."""
    test_sr = """FEMISAN A
    NAJBOLJE JUTRO
    07-09h
    Svakog radnog dana od 07h do 09h prvu, drugu i trecu jutarnju kafu sa Vama pije ekipa Najboljeg Jutra.
    Zapocnite jutro uz najvece hitove za celu Srbiju, uz zanimljive teme i puno smeha.
    """
    assert lc.get_winner(ALL_TERM_RANKS, ALL_CHAR_WEIGHTS, test_sr) == "hr"



from lplangid import count_utils as cu
import pytest


def test_normalize_score_dict():
    assert cu.normalize_score_dict({"foo": 1, "bar": 2}) == {"foo": 1/3, "bar": 2/3}
    assert cu.normalize_score_dict({"foo": 0, "bar": 0}) == {"foo": 1/2, "bar": 1/2}

    # L1 normalization is idempotent.
    assert cu.normalize_score_dict({"foo": 1, "bar": 2}) == cu.normalize_score_dict(
        cu.normalize_score_dict({"foo": 1/3, "bar": 2/3}))


def test_softmax_score_dict():
    assert cu.softmax_score_dict({"foo": 1, "bar": 2}) == pytest.approx({'bar': 0.731058578, 'foo': 0.268941421})
    assert cu.softmax_score_dict({"foo": 2, "bar": 3}) == pytest.approx({'bar': 0.731058578, 'foo': 0.268941421})
    assert cu.softmax_score_dict({"foo": 10, "bar": 11}) == pytest.approx({'bar': 0.731058578, 'foo': 0.268941421})
    assert cu.softmax_score_dict({"foo": 0, "bar": 0}) == {"foo": 1/2, "bar": 1/2}

    # Note that softmax normalization is not idempotent.
    assert not cu.softmax_score_dict({"foo": 1, "bar": 2}) == cu.softmax_score_dict(
        cu.softmax_score_dict({"foo": 1, "bar": 2}))


def test_softmax_score_dict_inverse():
    assert (cu.softmax_to_l1(cu.softmax_score_dict({"foo": 1/3, "bar": 2/3}))
            == pytest.approx({"foo": 1/3, "bar": 2/3}))
    initial = {"foo": 1, "bar": 2}
    softmaxed = cu.softmax_score_dict(initial)
    assert cu.softmax_to_l1(cu.softmax_score_dict(softmaxed)) == pytest.approx(softmaxed)


def test_freq_dict_to_lowercase():
    freq_dict = {"OK": 3, "ok": 4, "Okay": 5}
    freq_dict = cu.freq_dict_to_lowercase(freq_dict)
    assert freq_dict == {"ok": 7, "okay": 5}


def test_freq_table_to_ranks():
    freq_dict = {"OK": 5, "ok": 4, "Okay": 3}
    assert cu.freq_table_to_ranks(freq_dict) == {"OK": 1, "ok": 2, "Okay": 3}


def test_precision_recall_f():
    assert cu.precision_recall_f(100, 80, 70) == pytest.approx((0.875, 0.7, 0.77778), abs=0.0001)
    assert cu.precision_recall_f(100, 80, 70, 0.5) == pytest.approx((0.875, 0.7, 0.83333), abs=0.0001)
    assert cu.precision_recall_f(800, 80, 70) == pytest.approx((0.875, 0.0875, 0.1590909), abs=0.0001)
    assert cu.precision_recall_f(100, 80, 70, 0.5) == pytest.approx((0.875, 0.7, 0.83333), abs=0.0001)



import re
import string
from typing import List


def tokenize_fast(input_text: str) -> List[str]:
    """Returns a very naive whitespace and punctuation based tokenization.

    This helps for most but not all languages, should only be used if you don't know the language yet,
    or if you have a lot of data and can sacrifice a lot of output quality for the sake of speed.
    """
    return strip_most_punctuation(remove_html_tags(input_text)).split()


def remove_html_tags(input_text: str) -> str:
    """Removes all text enclosed by angle brackets."""
    html_regex = re.compile("<.*?>")
    return re.sub(html_regex, "", input_text)


def strip_most_punctuation(input_text: str) -> str:
    """Removes most punctuation except for particular characters inside a word.

    E.g., "The dog." becomes "The dog" but "U.S.A." becomes "U.S.A".
    """
    chars = [c for c in input_text]
    for i in range(len(chars)):
        if chars[i] in string.punctuation:
            if ((chars[i] in "'./?&=:")
                    and 0 < i < len(chars) - 1 and not chars[i-1].isspace() and not chars[i+1].isspace()):
                continue
            chars[i] = ' '
    return ''.join(chars)



import glob
from setuptools import setup

setup(
    name='lplangid',  # Required.
    version='0.1.4',  # Required. Use major.minor.dev format.
    description='LivePerson language detection package using a Reciprocal Rank Classifier',  # Required
    long_description='This package is a python implementation of the classifier described in the paper '
                     'Language Identification with a Reciprocal Rank Classifier".',
    long_description_content_type='text/x-rst',

    packages=['lplangid', 'training'],
    include_package_data=True,
    data_files=[('lplangid/freq_data', glob.glob('lplangid/freq_data/*.csv'))],

    # No new packages are needed for running lplangid - the below are useful for development.
    # install_requires=['flake', 'pytest'],  # List new package requirements here, but please be sure you need them!

    url='https://github.com/LivePersonInc/lplangid',  # Optional
    author='Dominic Widdows, Chris Brew',  # Optional
    author_email='dwiddows@gmail.com,cbrew@acm.org',  # Optional
    keywords='language detection, language classification, reciprocal rank classifier, lplangid'
)
